/*******************************************************************************
 *	BdsDataClient4.cpp	BDS API example code for a Data Client
 *				T.Barnaby,	BEAM Ltd,	2015-06-09
 *******************************************************************************
 *
 *	This is a very basic example of using the BdsApi from a data access
 *	perspective. It just provides information on a given user ID.
 */
#include <iostream>
#include <iomanip>
#include <stdio.h>
#include <BdsD.h>
#include <BdsC.h>

using namespace Bds;
using namespace std;


// Function to read some data
BError bdsTest(DataAccess& bds){
	BError			err;
	User			user;
	BUInt32			id = 4;
	
	// Get list of all data available for the selection		
	if(err = bds.userGetFromId(id, user)){
		return err.set(1, BString("Error: Getting user information: ") + err.getString());
	}

	cout << "ID:           " << user.id << "\n";
	cout << "Login:        " << user.user << "\n";
	cout << "Name:         " << user.name << "\n";
	cout << "Email:        " << user.email << "\n";
	cout << "Telephone:    " << user.telephone << "\n";
	cout << "Address:      " << user.address << "\n";
	cout << "Enabled:      " << user.enabled << "\n";
	
	return err;
}

int main(int argc, char** argv){
	BError			err;
	BString			hostName;
	DataAccess		bds;

	hostName = getenv("BDS_HOST");
	if(hostName == "")
		hostName = "localhost";
	
	if(argc == 2)
		hostName = argv[1];

	// Connect to the DataAccess service
	if(err = bds.connectService(BString("//") + hostName + "/bdsDataAccess")){
		cerr << "Error: " << err.getString() << "\n";
		return 1;
	}
	
	// Connect to service
	if(!err)
		err = bds.connect("test", "beam00");
	
	// Run a normal data gathering as a normal data access client would.
	if(!err)
		err = bdsTest(bds);
	
	if(err){
		cerr << "Error: " << err.getString() << "\n";
		return 1;
	}
		
	return 0;
}
