#!/usr/bin/python
################################################################################
#	BdsMetaData4.py		BDS API example code for a BDS MetaData Client
#				T.Barnaby,	BEAM Ltd,	2016-04-22
################################################################################
#
# This example shows how to obtain some MetaData from the BDS system.
# The example assumes the BDS test data is installed.
#
# This program deletes all of the stations:channel entries from an arry type station.
#
import sys
import time
import copy
import getopt
from bdslib import *

# Function to read display info on Station
def bdsArrayDeleteStations(bds):
	err	= BError();
	selection = Selection();
	
	# Set up selection
	selection.startTime.setString("2008-01-01T00:00:00.000000");
	selection.endTime.setString("2008-01-01T00:01:00.000000");
	selection.channels.append(SelectionChannel("TT", "TSA", "", ""));
#	bdsDumpSelection(selection);

	# Get list of stations available
	(err, stations) = bds.stationGetList(selection);
	if(err):
		return err.set(1, "Error: Getting stations: " + err.getString());

	# This displays some of the information available
	for s in stations:
		print "Station: " + s.name + " Type: " + s.type;
		print "    " + "Description: " + s.description + " Number of station/channels " + str(s.channels.number());
		for c in s.channels:
			print "%s:%s" % (c.station, c.channel);

		# Delete one by adding all required ones to a new list
		l = BListArrayChannel();
		for c in s.channels:
			print "%s:%s" % (c.station, c.channel);
			if(c.station == "TSN09"):
				print "Delete: %s:%s" % (c.station, c.channel);
			else:
				l.append(c);
	 
	 	print "New list";
		s.channels = l;
		for c in s.channels:
			print "%s:%s" % (c.station, c.channel);
		

	# Assuming we have a station match, delete all station:channel pairs from this StationArray
	if(0 and len(stations) == 1):
		station = stations[0];			# Get the station onject to manipulate
		station.channels.clear();		# Clear its channels array
		bds.stationUpdate(0, station);		# Update station object in the BDS system

		print "Removed station/channels from StationArray: %s" % (station.name);
		
		station.channels.append(ArrayChannel("TSB01", "BHZ_01"));
		print "Added two station/channel pairs to the StationArray";
		bds.stationUpdate(0, station);		# Update station object in the BDS system
		
	return err;

def main():
	hostName = "localhost";
	
	# Create DataAccess object to connect to BDS Server
	bds = AdminAccess();

	# Connect to the DataAccess service
	err = bds.connectService("//" + hostName + "/bdsAdminAccess");
	if(err):
		print "Error: " + str(err) + "\n";
		return 1;

	# Connect to service
	err = bds.connect("testAdmin", "beam00");
	if(err):
		print "Error: " + str(err) + "\n";
		return 1;
	
	(err, version, name) = bds.getVersion();
	if(err):
		print "Error: " + str(err) + "\n";
		return 1;
	print "Version:" , version, "Name:", name;

	err = bdsArrayDeleteStations(bds);
	if(err):
		print "Error:", err.getErrorNo(), err.getString();
		return 1;
		
	return 0;

if __name__ == "__main__":
	main();
