/*******************************************************************************
 *	BdsD.h	Produced by Bidl
 *******************************************************************************
 */

#ifndef BDSD_H
#define BDSD_H 1

#include <Boap.h>
#include <BObj.h>
#include <BDate.h>
#include <BTimeStamp.h>
#include <BComplex.h>
#include <BList.h>
#include <BArray.h>

//*****************************************************************************/
//	BDS API
//			T.Barnaby,	BEAM Ltd,	2008-03-04
//*****************************************************************************/
//
namespace Bds {
	/// The System Error List
	enum Errors 	{ ErrorNoMetaData = 64, ErrorDataQuality = 65, ErrorSlaveMode = 66, ErrorTimeStamp = 67, ErrorValidate = 80, ErrorValidateMissingBlocks = 81, ErrorValidateTimeBackwards = 82, ErrorValidateFilenameTime = 83, ErrorValidateMetaData = 84, ErrorValidateFix = 85, ErrorValidateDuplicate = 86, ErrorValidateReorder = 87, ErrorValidateBdsFudge = 88};

	enum Priority 	{ PriorityLow, PriorityNormal, PriorityHigh};

	enum Mode 	{ ModeMaster, ModeSlave};

	enum DataFlags 	{ DataFlagNone = 0x00, DataFlagClipDataToTime = 0x01, DataFlagClipDataToChannels = 0x02, DataFlagMergeSegments = 0x04, DataFlagNoMetadata = 0x08};

	enum SelectionGroup 	{ SelectionGroupData, SelectionGroupMetaData, SelectionGroupDataWithCount};

	enum SampleFormat 	{ SampleFormatUnknown, SampleFormatInt16, SampleFormatInt32, SampleFormatFloat32, SampleFormatFloat64, SampleFormatInt24};

	enum AvailType 	{ AvailNone, AvailPartial, AvailFull};

	class Point {
	public:
		Point(BFloat64 px = 0, BFloat64 py = 0);
	public:
		BFloat64	x;
		BFloat64	y;
	};

	class TimePeriod : public BObj {
	public:
		TimePeriod(BTimeStamp pstartTime = BTimeStamp(), BTimeStamp pendTime = BTimeStamp());
		BString getType();
		BError setMembers(BDictString& members);
		BError setMember(BString name, BString value);
		BError getMembers(BDictString& members);
		BError getMember(BString name, BString& value);
	public:
		BTimeStamp	startTime;	///< The Start time to the nearest us
		BTimeStamp	endTime;	///< The End time to the nearest us
	};

	/// This class defines a TimePeriod
	class ListRange : public BObj {
	public:
		ListRange(BUInt32 pstart = 0, BUInt32 pnumber = 0, BInt32 preverse = 0);
		BString getType();
		BError setMembers(BDictString& members);
		BError setMember(BString name, BString value);
		BError getMembers(BDictString& members);
		BError getMember(BString name, BString& value);
	public:
		BUInt32	start;	///< The start position
		BUInt32	number;	///< The number of items
		BInt32	reverse;	///< List from end
	};

	/// This class defines a Network organisation. Could have other information here like website, contacts etc
	class Network : public BObj {
	public:
		Network(BUInt32 pid = 0, BString pnetwork = BString(), BString pdescription = BString(), BList<BString > pstations = BList<BString >());
		BString getType();
		BError setMembers(BDictString& members);
		BError setMember(BString name, BString value);
		BError getMembers(BDictString& members);
		BError getMember(BString name, BString& value);
	public:
		BUInt32	id;	///< ID
		BString	network;	///< The name
		BString	description;	///< The organisations description
		BList<BString >	stations;	///< The list of arrays/stations the Network uses
	};

	/// This class defines a Source
	class Source : public BObj {
	public:
		Source(BUInt32 pid = 0, BString psource = BString(), BString psourceMeta = BString(), BString palias = BString(), BString pdescription = BString());
		BString getType();
		BError setMembers(BDictString& members);
		BError setMember(BString name, BString value);
		BError getMembers(BDictString& members);
		BError getMember(BString name, BString& value);
	public:
		BUInt32	id;	///< ID
		BString	source;	///< The sensor data's source name
		BString	sourceMeta;	///< The associated metadata's source name
		BString	alias;	///< The short alias for data files
		BString	description;	///< The description
	};

	/// This class defines a Source Order entry
	class SourcePriority : public BObj {
	public:
		SourcePriority(BUInt32 pid = 0, BTimeStamp pstartTime = BTimeStamp(), BTimeStamp pendTime = BTimeStamp(), BString psource = BString(), BUInt32 ppriority = 0);
		BString getType();
		BError setMembers(BDictString& members);
		BError setMember(BString name, BString value);
		BError getMembers(BDictString& members);
		BError getMember(BString name, BString& value);
	public:
		BUInt32	id;	///< ID
		BTimeStamp	startTime;	///< The Start Time
		BTimeStamp	endTime;	///< The End Time the channel was available
		BString	source;	///< The source name
		BUInt32	priority;	///< The priority order, highest first
	};

	/// This class defines a full channel name
	class ChannelName {
	public:
		ChannelName(BString pnetwork = BString(), BString pstation = BString(), BString pchannel = BString(), BString psource = BString());
	public:
		BString	network;	///< The Channels network
		BString	station;	///< The Channels station.
		BString	channel;	///< The Channels name
		BString	source;	///< The Channels source.
	};

	class ArrayChannel {
	public:
		ArrayChannel(BString pstation = BString(), BString pchannel = BString());
	public:
		BString	station;	///< The Stations name.
		BString	channel;	///< The Channels name
	};

	class Station {
	public:
		Station(BUInt32 pid = 0, BString pname = BString(), BString palias = BString(), BString ptype = BString(), BString pdescription = BString(), BList<ArrayChannel > pchannels = BList<ArrayChannel >());
	public:
		BUInt32	id;	///< ID
		BString	name;	///< The name
		BString	alias;	///< Alias name to be returned to the user
		BString	type;	///< Array or Station
		BString	description;	///< Description
		BList<ArrayChannel >	channels;	///< List of channels if an Array.
	};

	/// This class defines a Location. This will be the stations location.
	class Location : public BObj {
	public:
		Location(BUInt32 pid = 0, BTimeStamp pstartTime = BTimeStamp(), BTimeStamp pendTime = BTimeStamp(), BString pnetwork = BString(), BString pstation = BString(), BString pdatum = BString(), BFloat64 plongitude = 0, BFloat64 platitude = 0, BFloat64 pelevation = 0, BFloat64 parrayOffsetEast = 0, BFloat64 parrayOffsetNorth = 0);
		BString getType();
		BError setMembers(BDictString& members);
		BError setMember(BString name, BString value);
		BError getMembers(BDictString& members);
		BError getMember(BString name, BString& value);
	public:
		BUInt32	id;	///< The ID
		BTimeStamp	startTime;	///< The Start Time
		BTimeStamp	endTime;	///< The End Time the channel was available
		BString	network;	///< The Network/Organisation Name
		BString	station;	///< The station this location is for
		BString	datum;	///< The locations Datum
		BFloat64	longitude;	///< The longitude in degrees using the WGS84 datum.
		BFloat64	latitude;	///< The Latitude in degrees using the WGS84 datum.
		BFloat64	elevation;	///< The ground level elevation in meters from the WGS84 ellipsoid (Sea level)
		BFloat64	arrayOffsetEast;	///< The Array offset in in an array in an easterly direction
		BFloat64	arrayOffsetNorth;	///< The Array offset in in an array in a northerly direction
	};

	/// This class defines a Pole/Zero Response
	class PoleZero {
	public:
		PoleZero(BArray<BComplex > ppoles = BArray<BComplex >(), BArray<BComplex > pzeros = BArray<BComplex >());
	public:
		BArray<BComplex >	poles;	///< Poles
		BArray<BComplex >	zeros;	///< Zeros
	};

	/// This class defines a Amplitude/Phase Response
	class Fap {
	public:
		Fap(BFloat64 pfrequency = 0, BFloat64 pamplitude = 0, BFloat64 pphase = 0);
	public:
		BFloat64	frequency;	///< The frequency
		BFloat64	amplitude;	///< The Amplitude
		BFloat64	phase;	///< The Phase
	};

	/// This class defines a Pole/Zero Response
	class FirEntry {
	public:
		FirEntry(BFloat64 pcoefficient = 0, BFloat64 perror = 0);
	public:
		BFloat64	coefficient;	///< Value
		BFloat64	error;	///< Error
	};

	/// This class defines a Pole/Zero Response
	class Fir {
	public:
		Fir(BArray<FirEntry > pb = BArray<FirEntry >(), BArray<FirEntry > pa = BArray<FirEntry >());
	public:
		BArray<FirEntry >	b;	///< Numerator
		BArray<FirEntry >	a;	///< Denominator
	};

	/// This class defines a Response
	class Response {
	public:
		Response(BUInt32 pid = 0, BTimeStamp pstartTime = BTimeStamp(), BTimeStamp pendTime = BTimeStamp(), BString pnetwork = BString(), BString pstation = BString(), BString pchannel = BString(), BString psource = BString(), BUInt32 pstage = 0, BString pname = BString(), BString ptype = BString(), PoleZero ppoleZeros = PoleZero(), BArray<Fap > pfaps = BArray<Fap >(), Fir pfir = Fir(), BFloat64 pgain = 0, BFloat64 pgainFrequency = 0, BString pstageType = BString(), BFloat64 pdecimation = 0, BString psymmetry = BString(), BString pdescription = BString(), BInt32 pmeasured = 0, BFloat64 psampleRate = 0);
	public:
		BUInt32	id;	///< The ID
		BTimeStamp	startTime;	///< The Start Time
		BTimeStamp	endTime;	///< The End Time the channel was available
		BString	network;	///< The Network/Organisation Name
		BString	station;	///< The station
		BString	channel;	///< The channel
		BString	source;	///< The source
		BUInt32	stage;	///< The stage
		BString	name;	///< The response name. (Anti-Aliasing filter, Digitiser, post filter etc)
		BString	type;	///< The type of response (PoleZero,AmplitudePhase or FIR Coefficients)
		PoleZero	poleZeros;	///< PoleZero, AmplitudePhase or FIR Coefficient data
		BArray<Fap >	faps;	///< The Frequency/Amplitude/Phase table
		Fir	fir;	///< The FIR filters coefficients
		BFloat64	gain;	///< Overall gain at gain frequency. (For information) (Do we need this ?)
		BFloat64	gainFrequency;	///< Frequency that gain is valid for. (For information) (Do we need this ?)
		BString	stageType;	///< The stage type: A - Analog (rad/sec), B - Analog (Hz), C - Composite, D - Digital
		BFloat64	decimation;	///< Decimation performed post filter
		BString	symmetry;	///< Symmetry for FIR coefficients (A = asymmetric, B = symmetric[odd], C = symmetric[even]) ??
		BString	description;	///< Misc description
		BInt32	measured;	///< If response was a measured response
		BFloat64	sampleRate;	///< The stage's sample rate
	};

	/// This class defines a calibration setting
	class Calibration : public BObj {
	public:
		Calibration(BUInt32 pid = 0, BTimeStamp pstartTime = BTimeStamp(), BTimeStamp pendTime = BTimeStamp(), BString pnetwork = BString(), BString pstation = BString(), BString pchannel = BString(), BString psource = BString(), BString pname = BString(), BFloat64 psamplingFrequency = 0, BFloat64 pcalibrationFrequency = 0, BFloat64 pcalibrationFactor = 0, BString pcalibrationUnits = BString(), BFloat64 pdepth = 0, BFloat64 phorizontalAngle = 0, BFloat64 pverticalAngle = 0);
		BString getType();
		BError setMembers(BDictString& members);
		BError setMember(BString name, BString value);
		BError getMembers(BDictString& members);
		BError getMember(BString name, BString& value);
	public:
		BUInt32	id;	///< The ID
		BTimeStamp	startTime;	///< The Start Time
		BTimeStamp	endTime;	///< The End Time
		BString	network;	///< The Network/Organisation Name
		BString	station;	///< The station
		BString	channel;	///< The channel
		BString	source;	///< The source
		BString	name;	///< The Calibrations name, "Main", "Measured"
		BFloat64	samplingFrequency;	///< The sample rate used
		BFloat64	calibrationFrequency;	///< The frequency that the CalibrationFactor value is valid for.
		BFloat64	calibrationFactor;	///< The scaling value to apply to the data to normalise data to the units. This is a measured value at the calibration frequency.
		BString	calibrationUnits;	///< The measurment units
		BFloat64	depth;	///< The depth of the sensor below ground level in meters
		BFloat64	horizontalAngle;	///< The Sensors channel placement horizontal angle in degrees clockwise from north.
		BFloat64	verticalAngle;	///< The Sensors channel placement vertical angle in degrees degrees with zero = vertically up.
	};

	/// This class defines a Digitiser
	class Digitiser : public BObj {
	public:
		Digitiser(BUInt32 pid = 0, BTimeStamp pstartTime = BTimeStamp(), BTimeStamp pendTime = BTimeStamp(), BString pname = BString(), BString ptype = BString(), BString pserialNumber = BString(), BUInt32 pnumberChannels = 0, BFloat64 pbaseSamplingFrequency = 0, BFloat64 pinitialSamplingFrequency = 0, BFloat64 pgain = 0, BInt32 pshared = 0);
		BString getType();
		BError setMembers(BDictString& members);
		BError setMember(BString name, BString value);
		BError getMembers(BDictString& members);
		BError getMember(BString name, BString& value);
	public:
		BUInt32	id;	///< The ID
		BTimeStamp	startTime;	///< The Start Time
		BTimeStamp	endTime;	///< The End Time the channel was available
		BString	name;	///< The Digitisers name
		BString	type;	///< The Digitisers type
		BString	serialNumber;	///< The digitisers's serial number.
		BUInt32	numberChannels;	///< The number of supported channels
		BFloat64	baseSamplingFrequency;	///< The base sampling frequency
		BFloat64	initialSamplingFrequency;	///< The initial pre-decimation sampling frequency
		BFloat64	gain;	///< The overall gain of the digitiser at the manufacturers calibration frequency. (For information only)
		BInt32	shared;	///< This digitiser is shared
	};

	/// This class defines a Sensor
	class Sensor : public BObj {
	public:
		Sensor(BUInt32 pid = 0, BTimeStamp pstartTime = BTimeStamp(), BTimeStamp pendTime = BTimeStamp(), BString pname = BString(), BString ptype = BString(), BString pserialNumber = BString(), BUInt32 pnumberChannels = 0, BString pgainUnits = BString(), BFloat64 pgain = 0, BUInt32 poldId = 0, BInt32 pshared = 0);
		BString getType();
		BError setMembers(BDictString& members);
		BError setMember(BString name, BString value);
		BError getMembers(BDictString& members);
		BError getMember(BString name, BString& value);
	public:
		BUInt32	id;	///< The ID
		BTimeStamp	startTime;	///< The Start Time
		BTimeStamp	endTime;	///< The End Time
		BString	name;	///< The Sensors name
		BString	type;	///< The type of sensor. (Seismometer, Hydrophone etc)
		BString	serialNumber;	///< The sensor's serial number. Only used when there is a unique physical sensor.
		BUInt32	numberChannels;	///< The number of supported channels
		BString	gainUnits;	///< The gain units
		BFloat64	gain;	///< The overall gain of the sensor at the manufacturers calibration frequency. (For information only)
		BUInt32	oldId;	///< The Id from the old Autodrm database
		BInt32	shared;	///< This sensor is shared
	};

	/// This class defines a Channel to Digitiser/Sensor linkage
	class ChannelInstrument : public BObj {
	public:
		ChannelInstrument(BUInt32 pid = 0, BTimeStamp pstartTime = BTimeStamp(), BTimeStamp pendTime = BTimeStamp(), BUInt32 pchannelId = 0, BString psource = BString(), BUInt32 pdigitiserId = 0, BUInt32 psensorId = 0);
		BString getType();
		BError setMembers(BDictString& members);
		BError setMember(BString name, BString value);
		BError getMembers(BDictString& members);
		BError getMember(BString name, BString& value);
	public:
		BUInt32	id;	///< ID
		BTimeStamp	startTime;	///< The Start Time
		BTimeStamp	endTime;	///< The End Time the channel was available
		BUInt32	channelId;	///< The channels Id
		BString	source;	///< The source
		BUInt32	digitiserId;	///< The Digitiser in use
		BUInt32	sensorId;	///< The sensor in use
	};

	/// This class defines a Channel
	class Channel : public BObj {
	public:
		Channel(BUInt32 pid = 0, BTimeStamp pstartTime = BTimeStamp(), BTimeStamp pendTime = BTimeStamp(), BString pnetwork = BString(), BString pstation = BString(), BString pchannel = BString(), BString pchannelType = BString(), BString pchannelAux = BString(), BString pdataType = BString(), BString pdescription = BString());
		BString getType();
		BError setMembers(BDictString& members);
		BError setMember(BString name, BString value);
		BError getMembers(BDictString& members);
		BError getMember(BString name, BString& value);
	public:
		BUInt32	id;	///< ID
		BTimeStamp	startTime;	///< The Start Time
		BTimeStamp	endTime;	///< The End Time the channel was available
		BString	network;	///< The Network Name
		BString	station;	///< The Stations name. (Do we want this in here and if so do we want, network, source etc)
		BString	channel;	///< The channels name
		BString	channelType;	///< The channels type
		BString	channelAux;	///< The channels auxilary identifier
		BString	dataType;	///< The Type of data
		BString	description;	///< The channels description
	};

	/// This class defines the set of data selection critera
	class SelectionInfo {
	public:
		SelectionInfo(BTimeStamp pstartTime = BTimeStamp(), BTimeStamp pendTime = BTimeStamp(), BList<BString > pnetworks = BList<BString >(), BList<BString > parrays = BList<BString >(), BList<BString > pstations = BList<BString >(), BList<BString > parraysAndStations = BList<BString >(), BList<BString > pchannels = BList<BString >(), BList<BString > psources = BList<BString >(), BUInt32 pnumDataChannels = 0);
	public:
		BTimeStamp	startTime;	///< The Start Time
		BTimeStamp	endTime;	///< The End Time
		BList<BString >	networks;	///< The list of Network Names
		BList<BString >	arrays;	///< The list of Array names
		BList<BString >	stations;	///< The list of Station names
		BList<BString >	arraysAndStations;	///< The list of Array and Station names
		BList<BString >	channels;	///< The list of Channels
		BList<BString >	sources;	///< The list of Data Sources
		BUInt32	numDataChannels;	///< The number of sets of data in the system
	};

	class SelectionChannel {
	public:
		SelectionChannel(BString pnetwork = BString(), BString pstation = BString(), BString pchannel = BString(), BString psource = BString());
	public:
		BString	network;
		BString	station;
		BString	channel;
		BString	source;
	};

	class Selection {
	public:
		Selection(BUInt32 pid = 0, ListRange prange = ListRange(), BTimeStamp pstartTime = BTimeStamp(), BTimeStamp pendTime = BTimeStamp(), BList<SelectionChannel > pchannels = BList<SelectionChannel >(), BUInt32 pchannelId = 0, BUInt32 pdigitiserId = 0, BUInt32 psensorId = 0, BUInt32 psensorOldId = 0, BInt32 pcompleteSegments = 0, BString pcalibrationName = BString());
	public:
		BUInt32	id;	///< The ID of the record to return
		ListRange	range;	///< The range of data to return
		BTimeStamp	startTime;	///< The Start Time
		BTimeStamp	endTime;	///< The End Time
		BList<SelectionChannel >	channels;	///< The data channels to select
		BUInt32	channelId;	///< The Channel id
		BUInt32	digitiserId;	///< The Digitiser id
		BUInt32	sensorId;	///< The Sensor id
		BUInt32	sensorOldId;	///< The Sensor old id
		BInt32	completeSegments;	///< Do not clip the segment times to match the required time period
		BString	calibrationName;	///< Calibration name to use
	};

	class ChannelInfo {
	public:
		ChannelInfo(BTimeStamp pstartTime = BTimeStamp(), BTimeStamp pendTime = BTimeStamp(), Station pstation = Station(), Location plocation = Location(), Channel pchannel = Channel(), BString psource = BString(), BString pdataType = BString(), Digitiser pdigitiser = Digitiser(), Sensor psensor = Sensor(), Calibration pcalibration = Calibration(), BList<Response > presponses = BList<Response >());
	public:
		BTimeStamp	startTime;	///< The Start Time
		BTimeStamp	endTime;	///< The End Time
		Station	station;	///< The Station info
		Location	location;	///< The Station location
		Channel	channel;	///< The Channel data
		BString	source;	///< The data source
		BString	dataType;	///< The DataType
		Digitiser	digitiser;	///< The Digitiser in use
		Sensor	sensor;	///< The Sensor in use
		Calibration	calibration;	///< The Calibration info
		BList<Response >	responses;	///< The list of frequency responses
	};

	/// This class provides information on a set of channels
	class ChannelInfos {
	public:
		ChannelInfos(BArray<BArray<ChannelInfo > > pchannels = BArray<BArray<ChannelInfo > >());
	public:
		BArray<BArray<ChannelInfo > >	channels;
	};

	/// This class defines information on a data file
	class DataFileInfo : public BObj {
	public:
		DataFileInfo(BUInt32 pid = 0, BTimeStamp pstartTime = BTimeStamp(), BTimeStamp pendTime = BTimeStamp(), BString plocation = BString(), BString pformat = BString(), BString purl = BString(), BString pcomment = BString(), BUInt32 pimportUserId = 0, BTimeStamp pimportTime = BTimeStamp(), BString pstate = BString());
		BString getType();
		BError setMembers(BDictString& members);
		BError setMember(BString name, BString value);
		BError getMembers(BDictString& members);
		BError getMember(BString name, BString& value);
	public:
		BUInt32	id;	///< ID
		BTimeStamp	startTime;	///< The Start Time
		BTimeStamp	endTime;	///< The End Time
		BString	location;	///< The storage location
		BString	format;	///< The data format
		BString	url;	///< The URL for file access
		BString	comment;	///< A comment on the file
		BUInt32	importUserId;	///< The user ID of the importing user
		BTimeStamp	importTime;	///< The Time the data was imported
		BString	state;	///< Status info on the import (importing, ok etc)
	};

	class DataChannel : public BObj {
	public:
		DataChannel(BUInt32 pid = 0, BTimeStamp pstartTime = BTimeStamp(), BTimeStamp pendTime = BTimeStamp(), BString pnetwork = BString(), BString pstation = BString(), BString pchannel = BString(), BString psource = BString(), BUInt32 pnumBlocks = 0, BUInt64 pnumSamples = 0, BFloat64 psampleRate = 0, BUInt32 psampleFormat = 0, BUInt32 pdataFileId = 0, BUInt32 pdataFileChannel = 0, BString pimportFormat = BString(), BString pimportFilename = BString(), BTimeStamp pimportStartTime = BTimeStamp(), BDict<BString > pinfo = BDict<BString >());
		BString getType();
		BError setMembers(BDictString& members);
		BError setMember(BString name, BString value);
		BError getMembers(BDictString& members);
		BError getMember(BString name, BString& value);
	public:
		BUInt32	id;	///< ID
		BTimeStamp	startTime;	///< The Start Time
		BTimeStamp	endTime;	///< The End Time
		BString	network;	///< The Network Name
		BString	station;	///< The Station name
		BString	channel;	///< The Channels name
		BString	source;	///< The Data Source
		BUInt32	numBlocks;	///< The total number of blocks per channel if known, 0 otherwise
		BUInt64	numSamples;	///< The total number of samples per channel if known, 0 otherwise
		BFloat64	sampleRate;	///< The data's sample rate
		BUInt32	sampleFormat;	///< The data sample format
		BUInt32	dataFileId;	///< The Data File Id
		BUInt32	dataFileChannel;	///< The Data File Channel number
		BString	importFormat;	///< The original data format
		BString	importFilename;	///< The original data file name
		BTimeStamp	importStartTime;	///< The original import files start time
		BDict<BString >	info;	///< Extra info on the channel
	};

	class DataInfo {
	public:
		DataInfo(BTimeStamp pstartTime = BTimeStamp(), BTimeStamp pendTime = BTimeStamp(), BString parray = BString(), BString pdescription = BString(), BUInt32 psynchronous = 0, BArray<BArray<DataChannel > > pchannels = BArray<BArray<DataChannel > >(), BDict<BString > pinfo = BDict<BString >(), BDict<BString > pinfoExtra = BDict<BString >(), BList<BString > pwarnings = BList<BString >());
	public:
		BTimeStamp	startTime;	///< The Start Time
		BTimeStamp	endTime;	///< The End Time
		BString	array;	///< The Seismic Array that all of the channels are from, if just one
		BString	description;	///< The Comment
		BUInt32	synchronous;	///< The channels are synchronously sampled
		BArray<BArray<DataChannel > >	channels;	///< The Data channels. Each channel can have multiple segments of data
		BDict<BString >	info;	///< Info on the set of channels
		BDict<BString >	infoExtra;	///< Extra Info on the set of channels. Used for extended error/logging information
		BList<BString >	warnings;	///< Warnings on the data set
	};

	/// This class defines information on a set of data
	class DataAvail {
	public:
		DataAvail(BTimeStamp pstartTime = BTimeStamp(), BTimeStamp pendTime = BTimeStamp(), AvailType pavailType = AvailType());
	public:
		BTimeStamp	startTime;	///< The Start Time
		BTimeStamp	endTime;	///< The End Time
		AvailType	availType;
	};

	/// This class defines information on a set of data
	class DataAvailChan {
	public:
		DataAvailChan(BTimeStamp pstartTime = BTimeStamp(), BTimeStamp pendTime = BTimeStamp(), BString pnetwork = BString(), BString pstation = BString(), BString pchannel = BString(), BString psource = BString(), BArray<DataAvail > psegments = BArray<DataAvail >());
	public:
		BTimeStamp	startTime;	///< The Start Time
		BTimeStamp	endTime;	///< The End Time
		BString	network;	///< The Network Name
		BString	station;	///< The Station name
		BString	channel;	///< The Channels name
		BString	source;	///< The Data Source
		BArray<DataAvail >	segments;	///< Segment info
	};

	/// This defines a handle to a data stream/file
	class DataHandle {
	public:
		DataHandle(BUInt32 phandle = 0, BUInt32 pdataFileId = 0);
	public:
		BUInt32	handle;	///< Opaque file handle
		BUInt32	dataFileId;	///< The data file ID if opened for write
	};

	class DataBlock {
	public:
		DataBlock(BTimeStamp pstartTime = BTimeStamp(), BTimeStamp pendTime = BTimeStamp(), BUInt32 pchannelNumber = 0, BUInt32 psegmentNumber = 0, BArray<BArray<BFloat64 > > pchannelData = BArray<BArray<BFloat64 > >(), BDict<BString > pinfo = BDict<BString >());
	public:
		BTimeStamp	startTime;	///< The Start Time
		BTimeStamp	endTime;	///< The End Time the channel was available
		BUInt32	channelNumber;	///< The first channel number
		BUInt32	segmentNumber;	///< The segment number
		BArray<BArray<BFloat64 > >	channelData;	///< The raw channel data in a 2 dimensional array, ordered as per channel information in dataInfo
		BDict<BString >	info;	///< Extra information on data or ASCII data
	};

	/// This holds information on a user
	class User : public BObj {
	public:
		User(BUInt32 pid = 0, BString puser = BString(), BString ppassword = BString(), BString pname = BString(), BString pemail = BString(), BString ptelephone = BString(), BString paddress = BString(), BInt32 penabled = 0, BList<BString > pgroups = BList<BString >());
		BString getType();
		BError setMembers(BDictString& members);
		BError setMember(BString name, BString value);
		BError getMembers(BDictString& members);
		BError getMember(BString name, BString& value);
	public:
		BUInt32	id;	///< The unique user ID
		BString	user;	///< The User ID
		BString	password;	///< The Users password
		BString	name;	///< The Users full name
		BString	email;	///< The users email Address
		BString	telephone;	///< The Users telephone number
		BString	address;	///< The Users postal address
		BInt32	enabled;	///< Whether the users account is enabled
		BList<BString >	groups;	///< The security groups the user belongs to
	};

	/// This holds information on a groups
	class Group : public BObj {
	public:
		Group(BUInt32 pid = 0, BString pgroup = BString(), BString pdescription = BString());
		BString getType();
		BError setMembers(BDictString& members);
		BError setMember(BString name, BString value);
		BError getMembers(BDictString& members);
		BError getMember(BString name, BString& value);
	public:
		BUInt32	id;	///< The unique id
		BString	group;	///< The Group name
		BString	description;	///< The Groups description
	};

	/// This holds information on data access groups
	class AccessGroup : public BObj {
	public:
		AccessGroup(BUInt32 pid = 0, BString pgroup = BString(), BTimeStamp pstartTime = BTimeStamp(), BTimeStamp pendTime = BTimeStamp(), BString pnetwork = BString(), BString pstation = BString());
		BString getType();
		BError setMembers(BDictString& members);
		BError setMember(BString name, BString value);
		BError getMembers(BDictString& members);
		BError getMember(BString name, BString& value);
	public:
		BUInt32	id;	///< The unique id
		BString	group;	///< The Group name
		BTimeStamp	startTime;	///< The Start Time
		BTimeStamp	endTime;	///< The End Time
		BString	network;	///< The Network Name
		BString	station;	///< The Station name
	};

	/// This holds information on a change
	class Change : public BObj {
	public:
		Change(BUInt32 pid = 0, BUInt32 pchangeGroupId = 0, BTimeStamp ptime = BTimeStamp(), BString ptype = BString(), BString ptable = BString(), BUInt32 prowId = 0);
		BString getType();
		BError setMembers(BDictString& members);
		BError setMember(BString name, BString value);
		BError getMembers(BDictString& members);
		BError getMember(BString name, BString& value);
	public:
		BUInt32	id;	///< The unique id
		BUInt32	changeGroupId;	///< The Change group ID
		BTimeStamp	time;	///< The Time the change was made
		BString	type;	///< The change type
		BString	table;	///< The database table affected
		BUInt32	rowId;	///< The database row affected
	};

	/// This holds information on a change
	class ChangeGroup : public BObj {
	public:
		ChangeGroup(BUInt32 pid = 0, BTimeStamp ptime = BTimeStamp(), BString ptype = BString(), BString puser = BString(), BString ptitle = BString(), BString pdescription = BString());
		BString getType();
		BError setMembers(BDictString& members);
		BError setMember(BString name, BString value);
		BError getMembers(BDictString& members);
		BError getMember(BString name, BString& value);
	public:
		BUInt32	id;	///< The unique id
		BTimeStamp	time;	///< The Time the change was made
		BString	type;	///< The type of change
		BString	user;	///< The user who made the change
		BString	title;	///< The Changes title
		BString	description;	///< The Description of the change
	};

	/// This holds information on a Note
	class Note : public BObj {
	public:
		Note(BUInt32 pid = 0, BTimeStamp pstartTime = BTimeStamp(), BTimeStamp pendTime = BTimeStamp(), BString pnetwork = BString(), BString pstation = BString(), BString pchannel = BString(), BString psource = BString(), BString ptype = BString(), BString puser = BString(), BTimeStamp ptimeAdded = BTimeStamp(), BInt32 perrorNumber = 0, BString ptitle = BString(), BString pdescription = BString(), BString pdocFormat = BString(), BString pdocUrl = BString(), BUInt32 pdataFileId = 0, BString pimportFilename = BString());
		BString getType();
		BError setMembers(BDictString& members);
		BError setMember(BString name, BString value);
		BError getMembers(BDictString& members);
		BError getMember(BString name, BString& value);
	public:
		BUInt32	id;	///< The unique id
		BTimeStamp	startTime;	///< The Start Time note is for
		BTimeStamp	endTime;	///< The End Time note is for
		BString	network;	///< The Network Name
		BString	station;	///< The Station/Array name
		BString	channel;	///< The Channels name
		BString	source;	///< The Data Source
		BString	type;	///< The Type (note, warning, error ...)
		BString	user;	///< The user
		BTimeStamp	timeAdded;	///< The Time Entered
		BInt32	errorNumber;	///< Error number if error
		BString	title;	///< The title
		BString	description;	///< The Description
		BString	docFormat;	///< Document format if any
		BString	docUrl;	///< Document Url if any
		BUInt32	dataFileId;	///< The data file id associated with this note
		BString	importFilename;	///< The import filename
	};

	/// This holds information on a Note
	class Log : public BObj {
	public:
		Log(BUInt32 pid = 0, BTimeStamp ptime = BTimeStamp(), BString ptype = BString(), BUInt32 ppriority = 0, BString psubSystem = BString(), BString ptitle = BString(), BString pdescription = BString());
		BString getType();
		BError setMembers(BDictString& members);
		BError setMember(BString name, BString value);
		BError getMembers(BDictString& members);
		BError getMember(BString name, BString& value);
	public:
		BUInt32	id;	///< The unique id
		BTimeStamp	time;	///< The Time
		BString	type;	///< The Type
		BUInt32	priority;	///< The priority 0 to 5
		BString	subSystem;	///< The SubSystem
		BString	title;	///< The Changes title
		BString	description;	///< The Description of the change
	};

	/// This is used for requesting log entries
	class LogSelect {
	public:
		LogSelect(BTimeStamp pstartTime = BTimeStamp(), BString ptype = BString(), BUInt32 ppriority = 0, BString psubSystem = BString());
	public:
		BTimeStamp	startTime;	///< The start time
		BString	type;	///< The Type
		BUInt32	priority;	///< The priority 0 to 5
		BString	subSystem;	///< The SubSystem
	};

	/// This is the clean options
	class CleanOptions {
	public:
		CleanOptions(BInt32 plogs = 0, BInt32 pchanges = 0, BInt32 pdeletedFiles = 0);
	public:
		BInt32	logs;	///< Clean the Logs
		BInt32	changes;	///< Clean the changes
		BInt32	deletedFiles;	///< Clean deleted data files
	};

	/// This holds information on a data format
	class DataFormat {
	public:
		DataFormat(BList<BString > pnames = BList<BString >(), BString pdescription = BString(), BInt32 pdataRead = 0, BInt32 pdataWrite = 0, BInt32 pmetaDataRead = 0, BInt32 pmetaDataWrite = 0, BString pextension = BString());
	public:
		BList<BString >	names;	///< The format names
		BString	description;	///< The description
		BInt32	dataRead;	///< Ability to read data
		BInt32	dataWrite;	///< Ability to write data
		BInt32	metaDataRead;	///< MetaData read supported
		BInt32	metaDataWrite;	///< MetaData write supported
		BString	extension;	///< Default filename extension
	};

	/// This class defines information on a data channel
	class DataChannelInfo : public BObj {
	public:
		DataChannelInfo(BUInt32 pid = 0, BTimeStamp pstartTime = BTimeStamp(), BTimeStamp pendTime = BTimeStamp(), BString plocation = BString(), BString pformat = BString(), BString purl = BString(), BString pcomment = BString(), BUInt32 pimportUserId = 0, BTimeStamp pimportTime = BTimeStamp(), BString pstate = BString());
		BString getType();
		BError setMembers(BDictString& members);
		BError setMember(BString name, BString value);
		BError getMembers(BDictString& members);
		BError getMember(BString name, BString& value);
	public:
		BUInt32	id;	///< ID
		BTimeStamp	startTime;	///< The Start Time
		BTimeStamp	endTime;	///< The End Time
		BString	location;	///< The stoarge location
		BString	format;	///< The data format
		BString	url;	///< The URL for file access
		BString	comment;	///< A comment on the file
		BUInt32	importUserId;	///< The user ID of the importing user
		BTimeStamp	importTime;	///< The Time the data was imported
		BString	state;	///< Status info on the import (inProgress, complete etc)
	};

	//
	/// This is the DataAdd Access API interface
	//
	/// This is the AdminAccess API interface
	//
}

#endif
